# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

from copy import copy
from ...spec import spec
from ...register import check


# Module API

@check('extra-value')
def extra_value(errors, columns, row_number, state=None):
    for column in copy(columns):
        if 'header' not in column:
            # Add error
            message = spec['errors']['extra-value']['message']
            message = message.format(
                row_number=row_number,
                column_number=column['number'])
            errors.append({
                'code': 'extra-value',
                'message': message,
                'row-number': row_number,
                'column-number': column['number'],
            })
            # Remove column
            columns.remove(column)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

from goodtables import presets


# Test

def test_table():
    errors, tables = presets.table('data/valid.csv')
    assert len(errors) == 0
    assert len(tables) == 1



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import json
from ...spec import spec
from ...register import check


# Module API

@check('duplicate-row')
def duplicate_row(errors, columns, row_number, state):
    rindex = state.setdefault('rindex', {})
    pointer = hash(json.dumps(list(column.get('value') for column in columns)))
    references = rindex.setdefault(pointer, [])
    if references:
        # Add error
        message = spec['errors']['duplicate-row']['message']
        message = message.format(
            row_number=row_number,
            row_numbers=', '.join(map(str, references)))
        errors.append({
            'code': 'duplicate-row',
            'message': message,
            'row-number': row_number,
            'column-number': None,
        })
        # Clear columns
        del columns[:]
    references.append(row_number)
